# XV6 实用程序：pwd、touch、head、semtest 和混合调度

本仓库包含了为 XV6 操作系统实现的一些基本实用程序，包括 `pwd`、`touch`、`head`、`semtest` 以及一种新的 **混合调度机制**，该机制结合了优先级调度、SRTF（Shortest Remaining Time First）调度和时间片轮转调度。新调度器用于展示如何在 XV6 中实现动态调度策略，并在进程调度时考虑优先级、剩余时间、时间片和老化机制。

## 实用程序

### 1. `pwd`

`pwd` 实用程序用于打印当前工作目录。它演示了如何使用 `getcwd` 系统调用来获取当前工作目录。

- 使用 `getcwd` 系统调用获取当前工作目录。
- 处理错误并在系统调用失败时打印错误信息。

#### 示例：
```bash
$ pwd
/home/user
```

### 2. touch

touch 实用程序用于创建新文件或更新已存在文件的时间戳。它演示了如何使用 open 系统调用并配合 O_CREAT 标志来创建文件。

•	如果文件不存在，则创建新文件；如果文件已存在，则更新文件的时间戳。

•	使用 open 系统调用以读写模式打开文件，若文件不存在则创建文件。

示例：
```
$ touch newfile.txt
```

### 3. head

head 实用程序用于打印文件的前几行。它演示了如何使用 open 和 read 系统调用来读取文件内容。

实现要点：

•	打开文件并读取其内容。

•	打印文件的前 N 行（默认为 10 行）。

示例：

$ head somefile.txt

### 4. semtest

semtest 实用程序演示了如何使用信号量（Semaphore）来实现生产者-消费者问题。它使用了简单的信号量操作（sem_wait 和 sem_signal），并展示了进程间的同步和互斥。

实现要点：
	•	生产者进程向缓冲区中添加物品，消费者进程从缓冲区取出物品。
	•	使用信号量 empty、full 和 mutex 来同步生产者和消费者进程。
	•	使用 sem_wait 和 sem_signal 实现信号量的等待和通知机制。

示例：
```
$ semtest
Producer: produced item 0
Producer: produced item 1
Consumer: consumed item 0
Consumer: consumed item 1
...
```

### 5. 混合调度机制
`proc.c` 
关键代码：
```
for (p = proc; p < &proc[NPROC]; p++)
    {
      acquire(&p->lock);
      if (p->state != UNUSED)
      {
        nproc++;
      }
      if (p->state == RUNNABLE)
      {
        // 老化机制：检查是否需要提高优先级
        uint64 current_time = r_time();
        if (current_time - p->last_scheduled > AGING_THRESHOLD)
        {
          p->priority--; // 优先级提高（数值减小）
          if (p->priority < 0)
          {
            p->priority = 0; // 确保优先级不为负数
          }
        }

        // 优先级调度：找到优先级最低的进程
        if (p->priority < min_priority ||
            (p->priority == min_priority && p->remaining_time < min_remaining_time))
        {
          selected_proc = p;
          min_priority = p->priority;
          min_remaining_time = p->remaining_time;
        }
      }
      release(&p->lock);
    }

```

在 XV6 中实现了一个新的 混合调度机制，该机制结合了优先级调度、SRTF 调度和时间片轮转调度。此调度器动态选择进程，确保系统高效运行，且避免进程长时间处于未调度状态。调度机制的主要特点包括：

•	优先级调度：优先选择优先级较低（数值越小）且可运行的进程。

•	SRTF 调度：对于相同优先级的进程，选择剩余执行时间最短的进程。

•	时间片轮转：在优先级和剩余时间相同的情况下，按时间片轮转进行调度。

•	老化机制：进程在长时间未被调度时，优先级逐渐提高，避免饥饿现象。

•	调度器在选择进程时，首先考虑其优先级，然后是剩余时间，再根据时间片和老化机制进行调度。

•	使用 `priority`、`remaining_time`、`time_slice` 和 `last_scheduled` 等字段来控制进程的调度顺序。
注：可根据实际需求随意调节

•	采用 aging_threshold 来设置进程老化的时间阈值，保证长时间未调度的进程得到优先调度。

示例：
```
$ make qemu
```
通过 XV6 的仿真环境启动系统，调度器将根据混合调度机制进行调度。

# 如何运行
1.	克隆本仓库到您的机器上：

2.	使用 make qemu 启动 XV6 仿真环境并运行相应的程序。调度器将自动加载并基于混合调度机制调度进程：
```
make qemu
$usertests
```

