#include "types.h"
#include "riscv.h"
#include "defs.h"
#include "spinlock.h"
#include "sem.h"


// 初始化信号量，设置初值
void sem_init(struct semaphore *sem, int initial_value)
{
  sem->value = initial_value;
  initlock(&sem->lock, "semaphore_lock");
  sem->qsize = 0;
}

// 等待信号量，如果信号量值为 0，则阻塞
void sem_wait(struct semaphore *sem)
{
  acquire(&sem->lock); // 加锁
  while (sem->value == 0)
  {
    // 如果信号量值为 0，进入等待队列
    sleep(sem, &sem->lock);
  }
  sem->value--;        // 信号量值减 1
  release(&sem->lock); // 解锁
}

// 释放信号量，如果有等待的进程，唤醒它
void sem_signal(struct semaphore *sem)
{
  acquire(&sem->lock); // 加锁
  sem->value++;        // 信号量值加 1
  if (sem->qsize > 0)
  {
    // 唤醒一个等待的进程
    struct proc *p = sem->queue[--sem->qsize];
    p->state = RUNNABLE; // 设置进程状态为可运行
  }
  release(&sem->lock); // 解锁
}