#include "kernel/types.h"
#include "kernel/stat.h"
#include "kernel/fs.h"
#include "user/user.h"
#include "kernel/param.h"

char *strcat(char *dest, const char *src)
{
  char *d = dest;
  while (*d)
    d++; // 找到 dest 的结尾
  while ((*d++ = *src++))
    ; // 复制 src 到 dest
  return dest;
}

void get_path(char *path)
{
  char buf[512];
  char temp[MAXPATH];
  struct stat st;
  int fd = open("..", 0); // 打开父目录
  if (fd < 0)
  {
    printf("pwd: cannot open parent directory\n");
    return;
  }

  // 获取当前目录的 inode
  if (fstat(fd, &st) < 0)
  {
    printf("pwd: cannot stat parent directory\n");
    close(fd);
    return;
  }

  // 根目录 inode 是固定的
  if (st.ino == ROOTINO)
  {
    strcpy(path, "/");
    close(fd);
    return;
  }

  // 遍历父目录，找到当前目录名
  while (read(fd, buf, sizeof(buf)) > 0)
  {
    struct dirent *de = (struct dirent *)buf;
    for (int i = 0; i < sizeof(buf) / sizeof(struct dirent); i++, de++)
    {
      if (de->inum == st.ino)
      {
        strcpy(temp, path);
        strcpy(path, de->name);
        strcat(path, "/");
        strcat(path, temp);
        close(fd);
        return;
      }
    }
  }
  close(fd);
}

int main()
{
  char path[MAXPATH] = "";
  get_path(path);
  if (path[0] == 0)
  {
    printf("pwd: error determining current directory\n");
    exit(1);
  }
  printf("%s\n", path);
  exit(0);
}